# coding: utf-8

"""
    File Service v2

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 0.1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self


class ZitadelUserState(str, Enum):
    """
    Enum of all the possible states of a user account (from Zitadel).
    """

    """
    allowed enum values
    """
    USER_STATE_ACTIVE = 'USER_STATE_ACTIVE'
    USER_STATE_INACTIVE = 'USER_STATE_INACTIVE'
    USER_STATE_DELETED = 'USER_STATE_DELETED'
    USER_STATE_LOCKED = 'USER_STATE_LOCKED'
    USER_STATE_SUSPEND = 'USER_STATE_SUSPEND'
    USER_STATE_INITIAL = 'USER_STATE_INITIAL'
    USER_STATE_UNSPECIFIED = 'USER_STATE_UNSPECIFIED'

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of ZitadelUserState from a JSON string"""
        return cls(json.loads(json_str))
